@@ -1,6 +1,13 @@
+from collections import defaultdict
+from io import BytesIO
+import logging
+import os
+from typing import List, Literal, Optional, Union
+
 import mando
-from tsgettoolbox.odo import odo, resource
 import pandas as pd
+import requests
+import typic
 
 try:
     from mando.rst_text_formatter import RSTHelpFormatter as HelpFormatter
@@ -8,6 +15,117 @@ except ImportError:
     from argparse import RawTextHelpFormatter as HelpFormatter
 
 from tstoolbox import tsutils
+from tsgettoolbox import utils
+
+
+_settings_map = defaultdict(lambda: [{"metric": "", "english": ""}])
+
+# Preliminary or verified water levels, depending on availability.
+_settings_map["water_level"] = [
+    {"metric": "m", "english": "ft"},
+    "h",
+]
+
+# Air temperature as measured at the station.
+_settings_map["air_temperature"] = [
+    {"metric": "degC", "english": "degF"},
+    "h",
+]
+
+# Water temperature as measured at the station.
+_settings_map["water_temperature"] = [
+    {"metric": "degC", "english": "degF"},
+    "h",
+]
+
+# Wind speed, direction, and gusts as measured at the station.
+_settings_map["wind"] = [{"metric": "m/s", "english": "ft/s"}, "h"]
+
+# Barometric pressure as measured at the station.
+_settings_map["air_pressure"] = [
+    {"metric": "mb", "english": "mb"},
+    "h",
+]
+
+# Air Gap (distance between a bridge and the water's surface) at the
+# station.
+_settings_map["air_gap"] = [
+    {"metric": "m", "english": "ft"},
+    "h",
+]
+
+# The water's conductivity as measured at the station.
+_settings_map["conductivity"] = [
+    {"metric": "mS/cm", "english": "mS/cm"},
+    "h",
+]
+
+# Visibility from the station's visibility sensor. A measure of atmospheric
+# clarity.
+_settings_map["visibility"] = [
+    {"metric": "km", "english": "nm"},
+    "h",
+]
+
+# Relative humidity as measured at the station.
+_settings_map["humidity"] = [
+    {"metric": "percent", "english": "percent"},
+    "h",
+]
+
+# Salinity and specific gravity data for the station.
+_settings_map["salinity"] = [
+    {"metric": "PSU", "english": "PSU"},
+    "h",
+]
+
+# Verified hourly height water level data for the station.
+_settings_map["hourly_height"] = [
+    {"metric": "m", "english": "ft"},
+    "h",
+]
+
+# Verified high/low water level data for the station.
+_settings_map["high_low"] = [
+    {"metric": "m", "english": "ft"},
+    None,
+]
+
+# Verified daily mean water level data for the station.
+_settings_map["daily_mean"] = [
+    {"metric": "m", "english": "ft"},
+    None,
+]
+
+# Verified monthly mean water level data for the station.
+_settings_map["monthly_mean"] = [
+    {"metric": "m", "english": "ft"},
+    None,
+]
+
+# One minute water level data for the station.
+_settings_map["one_minute_water_level"] = [
+    {"metric": "m", "english": "ft"},
+    None,
+]
+
+# 6 minute predictions water level data for the station.
+_settings_map["predictions"] = [
+    {"metric": "m", "english": "ft"},
+    "h",
+]
+
+# datums data for the currents stations.
+_settings_map["datums"] = [
+    {"metric": "m", "english": "ft"},
+    None,
+]
+
+# Currents data for currents stations.
+_settings_map["currents"] = [
+    {"metric": "m/s", "english": "ft/s"},
+    "h",
+]
 
 
 @mando.command("coops", formatter_class=HelpFormatter, doctype="numpy")
@@ -26,7 +144,7 @@ def coops_cli(
 ):
     r"""Download Center for Operational Oceanographic Products and Services.
 
-    CO-OPS web services is at http://tidesandcurrents.noaa.gov/api/.
+    CO-OPS web services is at https://tidesandcurrents.noaa.gov/api/.
     The time zone of the returned data depends on the setting of the
     "time_zone" option.  The default is "GMT" also known as "UTC".
 
@@ -39,7 +157,7 @@ def coops_cli(
             Example: '--station=9414290'
 
         Station listings for various products can be viewed at
-        http://tidesandcurrents.noaa.gov or viewed on a map at Tides
+        https://tidesandcurrents.noaa.gov or viewed on a map at Tides
         & Currents Station Map
 
     date : str
@@ -518,82 +636,275 @@ def coops_cli(
     )
 
 
-@tsutils.validator(
-    station=[str, ["pass", []], 1],
-    date=[str, ["domain", ["latest", "today", "recent"]], 1],
-    begin_date=[tsutils.parsedate, ["pass", []], 1],
-    end_date=[tsutils.parsedate, ["pass", []], 1],
-    range=[int, ["range", [0, None]], 1],
-    product=[
-        str,
-        [
-            "domain",
-            [
-                "water_level",
-                "air_temperature",
-                "water_temperature",
-                "wind",
-                "air_gap",
-                "conductivity",
-                "visibility",
-                "humidity",
-                "salinity",
-                "hourly_height",
-                "high_low",
-                "daily_mean",
-                "monthly_mean",
-                "one_minute_water_level",
-                "predictions",
-                "datums",
-                "currents",
-            ],
-        ],
-        None,
-    ],
-    datum=[
-        str,
-        ["domain", ["MHHW", "MHW", "MTL", "MSL", "MLW", "MLLW", "NAVD", "STND"]],
-        1,
-    ],
-    time_zone=[
-        str,
-        ["domain", ["gmt", "lst", "lst_ldt", "GMT", "UTC", "utc", "LST", "LST_LDT"]],
-        1,
-    ],
+def core(params):
+    if params["end_date"] is not None:
+        params["end_date"] = params["end_date"].strftime("%Y%m%d")
+    if params["begin_date"] is not None:
+        params["begin_date"] = params["begin_date"].strftime("%Y%m%d")
+
+    url = r"https://tidesandcurrents.noaa.gov/api/datagetter"
+    session = utils.requests_retry_session()
+    req = session.get(url, params=params)
+
+    if os.path.exists("debug_tsgettoolbox"):
+        logging.warning(req.url)
+    req.raise_for_status()
+
+    if (
+        b"Error" in req.content
+        or b"Wrong" in req.content
+        or b"Range limit" in req.content
+    ):
+        df = pd.DataFrame()
+        error = req.content
+    else:
+        df = pd.read_csv(BytesIO(req.content), index_col=0, parse_dates=True)
+        error = ""
+    return df, error
+
+
+def precore(query_params, i):
+    if query_params["date"] is not None:
+        ndf, error = core(query_params)
+    elif (
+        query_params["range"] is not None
+        and query_params["begin_date"] is None
+        and query_params["end_date"] is None
+    ):
+        ndf, error = core(query_params)
+    elif query_params["begin_date"] is not None and query_params["range"] is not None:
+        ndf, error = core(query_params)
+    elif (
+        query_params["begin_date"] is not None and query_params["end_date"] is not None
+    ):
+        ndf = pd.DataFrame()
+        sdate = query_params["begin_date"]
+        edate = query_params["end_date"]
+        testdate = sdate
+        while testdate < edate:
+            query_params["begin_date"] = testdate
+            testdate = testdate + pd.Timedelta(days=deltas[i])
+            if testdate > edate:
+                testdate = edate
+            query_params["end_date"] = testdate
+            df, error = core(query_params)
+            ndf = ndf.combine_first(df)
+    elif query_params["end_date"] is not None and query_params["range"] is not None:
+        ndf, error = core(query_params)
+
+    if len(ndf) == 0:
+        raise ValueError(
+            tsutils.error_wrapper(
+                """
+COOPS service returned the error "{0}".
+""".format(
+                    error
+                )
+            )
+        )
+
+    new_column_names = []
+    for icolumn_name in ndf.columns:
+        ncolumn_name = icolumn_name.lower().strip().replace(" ", "_")
+        units = _settings_map[ncolumn_name][0][query_params["units"]]
+        unitstr = ncolumn_name
+        if units != "":
+            unitstr = ":".join([ncolumn_name, units])
+        new_column_names.append(("NOS", query_params["station"], unitstr))
+    ndf.columns = ["-".join(i).rstrip("-") for i in new_column_names]
+    time_zone_name = query_params["time_zone"].upper()
+    if time_zone_name == "GMT":
+        time_zone_name = "UTC"
+    ndf = ndf.tz_localize(time_zone_name)
+    ndf.index.name = "Datetime:{0}".format(time_zone_name)
+    return ndf
+
+
+deltas = {
+    "water_level": 31,
+    "air_temperature": 31,
+    "water_temperature": 31,
+    "wind": 31,
+    "air_gap": 31,
+    "conductivity": 31,
+    "visibility": 31,
+    "humidity": 31,
+    "salinity": 31,
+    "hourly_height": 365,
+    "high_low": 365,
+    "daily_mean": 3650,
+    "monthly_mean": 3650,
+    "one_minute_water_level": 31,
+    "predictions": 31,
+    "datums": 31,
+    "currents": 31,
+}
+
+
+@tsutils.transform_args(
+    product=tsutils.make_list,
+    time_zone=str.upper,
+    datum=str.upper,
+    begin_date=tsutils.parsedate,
+    end_date=tsutils.parsedate,
 )
+@typic.al
 def coops(
-    station,
-    date=None,
-    begin_date=None,
-    end_date=None,
-    range=None,
-    product="water_level",
-    datum="NAVD",
-    time_zone="GMT",
+    station: str,
+    date: Literal["latest", "today", "recent"] = None,
+    begin_date: Optional[pd.Timestamp] = None,
+    end_date: Optional[pd.Timestamp] = None,
+    range: Optional[int] = None,
+    product: List[
+        Literal[
+            "water_level",
+            "air_temperature",
+            "water_temperature",
+            "wind",
+            "air_gap",
+            "conductivity",
+            "visibility",
+            "humidity",
+            "salinity",
+            "hourly_height",
+            "high_low",
+            "daily_mean",
+            "monthly_mean",
+            "one_minute_water_level",
+            "predictions",
+            "datums",
+            "currents",
+        ]
+    ] = "water_level",
+    datum: Literal["MHHW", "MHW", "MTL", "MSL", "MLW", "MLLW", "NAVD", "STND"] = "NAVD",
+    time_zone: Literal["GMT", "UTC", "LST", "LST_LDT"] = "GMT",
     interval="h",
     bin=None,
 ):
     r"""Download Center for Operational Oceanographic Products and Services."""
-    from tsgettoolbox.services import coops as placeholder
+    params = {}
+    params["station"] = station
+    params["date"] = date
+    params["range"] = range
+    params["product"] = product
+    params["bin"] = bin
+    params["interval"] = interval
+    params["units"] = "metric"
+    params["time_zone"] = time_zone
+    params["datum"] = datum
+    try:
+        params["begin_date"] = begin_date
+    except ValueError:
+        params["begin_date"] = None
+    try:
+        params["end_date"] = end_date
+    except ValueError:
+        params["end_date"] = None
+    params["format"] = "csv"
+    params["application"] = "tsgettoolbox"
 
     ndf = pd.DataFrame()
-    for cnt, i in enumerate(tsutils.make_list(product)):
-        r = resource(
-            r"http://tidesandcurrents.noaa.gov/api/datagetter",
-            station=station,
-            date=date,
-            begin_date=begin_date,
-            end_date=end_date,
-            range=range,
-            product=i,
-            datum=datum,
-            units="metric",
-            time_zone=time_zone,
-            interval=interval,
-            bin=bin,
-        )
-        ndf = ndf.join(odo(r, pd.DataFrame), how="outer", rsuffix="_{0}".format(cnt))
+    for cnt, i in enumerate(product):
+        r = precore(params, i)
+        ndf = ndf.join(r, how="outer", rsuffix="_{0}".format(cnt))
     return ndf
 
 
 coops.__doc__ = coops_cli.__doc__
+
+
+if __name__ == "__main__":
+    """
+    https://tidesandcurrents.noaa.gov/api/datagetter?begin_date=20020101
+    &end_date=20020102&range=1&station=8720218&product=water_level
+    """
+
+    r = coops(
+        station="8720218",
+        product="water_level",
+        interval="h",
+        time_zone="gmt",
+        datum="mllw",
+        range=20,
+        begin_date=None,
+        end_date=None,
+        date=None,
+    )
+
+    print("tidesandcurrents")
+    print(r)
+
+    r = coops(
+        station="8720218",
+        product="water_temperature",
+        interval="h",
+        time_zone="gmt",
+        datum="mllw",
+        begin_date="01/10/2002",
+        range=2,
+        end_date=None,
+        date=None,
+    )
+
+    print("tidesandcurrents")
+    print(r)
+
+    r = coops(
+        station="8720218",
+        product="water_level",
+        interval="h",
+        time_zone="gmt",
+        datum="mllw",
+        begin_date="01/10/2002",
+        range=5,
+        end_date=None,
+        date=None,
+    )
+
+    print("tidesandcurrents")
+    print(r)
+
+    r = coops(
+        station="8720218",
+        product="air_temperature",
+        interval="h",
+        time_zone="gmt",
+        datum="mllw",
+        end_date="01/10/2002",
+        range=3,
+        begin_date=None,
+        date=None,
+    )
+
+    print("tidesandcurrents")
+    print(r)
+
+    r = coops(
+        station="8720218",
+        product="water_level",
+        interval="h",
+        time_zone="gmt",
+        datum="mllw",
+        begin_date="01/10/2002",
+        end_date="2003-01-01",
+        range=None,
+        date=None,
+    )
+
+    print("tidesandcurrents")
+    print(r)
+
+    try:
+        r = coops(
+            station="8720218",
+            product="water_level",
+            time_zone="gmt",
+            datum="mllw",
+            begin_date=None,
+            end_date=None,
+            range=745,
+            date=None,
+        )
+        print(r)
+    except ValueError:
+        print("Correct ValueError")
